@@ -1,6 +1,6 @@
 # encoding:utf-8
 #--
-# Addressable, Copyright (c) 2006-2008 Bob Aman
+# Addressable, Copyright (c) 2006-2010 Bob Aman
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -22,9 +22,6 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #++
 
-$:.unshift(File.expand_path(File.join(File.dirname(__FILE__), '/..')))
-$:.uniq!
-
 require "addressable/version"
 require "addressable/uri"
 
@@ -39,7 +36,7 @@ module Addressable
       Addressable::URI::CharacterClasses::UNRESERVED
     OPERATOR_EXPANSION =
       /\{-([a-zA-Z]+)\|([#{anything}]+)\|([#{anything}]+)\}/
-    VARIABLE_EXPANSION = /\{([#{anything}]+?)(=([#{anything}]+))?\}/
+    VARIABLE_EXPANSION = /\{([#{anything}]+?)(?:=([#{anything}]+))?\}/
 
     ##
     # Raised if an invalid template value is supplied.
@@ -154,21 +151,23 @@ module Addressable
     #
     # @param [Addressable::URI, #to_str] uri
     #   The URI to extract from.
+    #
     # @param [#restore, #match] processor
     #   A template processor object may optionally be supplied.
-    #   The object should respond to either the <tt>restore</tt> or
-    #   <tt>match</tt> messages or both.  The <tt>restore</tt> method should
-    #   take two parameters: [String] name and [String] value.  The
-    #   <tt>restore</tt> method should reverse any transformations that have
-    #   been performed on the value to ensure a valid URI.  The
-    #   <tt>match</tt> method should take a single parameter: [String] name.
-    #   The <tt>match</tt> method should return a <tt>String</tt> containing
-    #   a regular expression capture group for matching on that particular
-    #   variable.  The default value is ".*?".  The <tt>match</tt> method has
-    #   no effect on multivariate operator expansions.
+    #
+    # The object should respond to either the <tt>restore</tt> or
+    # <tt>match</tt> messages or both.  The <tt>restore</tt> method should take
+    # two parameters: [String] name and [String] value.  The <tt>restore</tt>
+    # method should reverse any transformations that have been performed on the
+    # value to ensure a valid URI.  The <tt>match</tt> method should take a
+    # single parameter: [String] name.  The <tt>match</tt> method should return
+    # a <tt>String</tt> containing a regular expression capture group for
+    # matching on that particular variable.  The default value is ".*?".  The
+    # <tt>match</tt> method has no effect on multivariate operator expansions.
+    #
     # @return [Hash, NilClass]
-    #   The <tt>Hash</tt> mapping that was extracted from the URI, or
-    #   <tt>nil</tt> if the URI didn't match the template.
+    # The <tt>Hash</tt> mapping that was extracted from the URI, or
+    # <tt>nil</tt> if the URI didn't match the template.
     #
     # @example
     #   class ExampleProcessor
@@ -212,21 +211,23 @@ module Addressable
     #
     # @param [Addressable::URI, #to_str] uri
     #   The URI to extract from.
+    #
     # @param [#restore, #match] processor
     #   A template processor object may optionally be supplied.
-    #   The object should respond to either the <tt>restore</tt> or
-    #   <tt>match</tt> messages or both.  The <tt>restore</tt> method should
-    #   take two parameters: [String] name and [String] value.  The
-    #   <tt>restore</tt> method should reverse any transformations that have
-    #   been performed on the value to ensure a valid URI.  The
-    #   <tt>match</tt> method should take a single parameter: [String] name.
-    #   The <tt>match</tt> method should return a <tt>String</tt> containing
-    #   a regular expression capture group for matching on that particular
-    #   variable.  The default value is ".*?".  The <tt>match</tt> method has
-    #   no effect on multivariate operator expansions.
+    #
+    # The object should respond to either the <tt>restore</tt> or
+    # <tt>match</tt> messages or both. The <tt>restore</tt> method should take
+    # two parameters: [String] name and [String] value. The <tt>restore</tt>
+    # method should reverse any transformations that have been performed on the
+    # value to ensure a valid URI. The <tt>match</tt> method should take a
+    # single parameter: [String] name. The <tt>match</tt> method should return
+    # a <tt>String</tt> containing a regular expression capture group for
+    # matching on that particular variable. The default value is ".*?". The
+    # <tt>match</tt> method has no effect on multivariate operator expansions.
+    #
     # @return [Hash, NilClass]
-    #   The <tt>Hash</tt> mapping that was extracted from the URI, or
-    #   <tt>nil</tt> if the URI didn't match the template.
+    # The <tt>Hash</tt> mapping that was extracted from the URI, or
+    # <tt>nil</tt> if the URI didn't match the template.
     #
     # @example
     #   class ExampleProcessor
@@ -325,19 +326,19 @@ module Addressable
     #
     # @param [Hash] mapping The mapping that corresponds to the pattern.
     # @param [#validate, #transform] processor
-    #   An optional processor object may be supplied.  The object should
-    #   respond to either the <tt>validate</tt> or <tt>transform</tt> messages
-    #   or both.  Both the <tt>validate</tt> and <tt>transform</tt> methods
-    #   should take two parameters: <tt>name</tt> and <tt>value</tt>.  The
-    #   <tt>validate</tt> method should return <tt>true</tt> or
-    #   <tt>false</tt>; <tt>true</tt> if the value of the variable is valid,
-    #   <tt>false</tt> otherwise.  An <tt>InvalidTemplateValueError</tt>
-    #   exception will be raised if the value is invalid.  The
-    #   <tt>transform</tt> method should return the transformed variable
-    #   value as a <tt>String</tt>.  If a <tt>transform</tt> method is used,
-    #   the value will not be percent encoded automatically.  Unicode
-    #   normalization will be performed both before and after sending the
-    #   value to the transform method.
+    #   An optional processor object may be supplied. 
+    #
+    # The object should respond to either the <tt>validate</tt> or
+    # <tt>transform</tt> messages or both. Both the <tt>validate</tt> and
+    # <tt>transform</tt> methods should take two parameters: <tt>name</tt> and
+    # <tt>value</tt>. The <tt>validate</tt> method should return <tt>true</tt>
+    # or <tt>false</tt>; <tt>true</tt> if the value of the variable is valid,
+    # <tt>false</tt> otherwise. An <tt>InvalidTemplateValueError</tt>
+    # exception will be raised if the value is invalid. The <tt>transform</tt>
+    # method should return the transformed variable value as a <tt>String</tt>.
+    # If a <tt>transform</tt> method is used, the value will not be percent
+    # encoded automatically. Unicode normalization will be performed both
+    # before and after sending the value to the transform method.
     #
     # @return [Addressable::Template] The partially expanded URI template.
     #
@@ -401,19 +402,19 @@ module Addressable
     #
     # @param [Hash] mapping The mapping that corresponds to the pattern.
     # @param [#validate, #transform] processor
-    #   An optional processor object may be supplied.  The object should
-    #   respond to either the <tt>validate</tt> or <tt>transform</tt> messages
-    #   or both.  Both the <tt>validate</tt> and <tt>transform</tt> methods
-    #   should take two parameters: <tt>name</tt> and <tt>value</tt>.  The
-    #   <tt>validate</tt> method should return <tt>true</tt> or
-    #   <tt>false</tt>; <tt>true</tt> if the value of the variable is valid,
-    #   <tt>false</tt> otherwise.  An <tt>InvalidTemplateValueError</tt>
-    #   exception will be raised if the value is invalid.  The
-    #   <tt>transform</tt> method should return the transformed variable
-    #   value as a <tt>String</tt>.  If a <tt>transform</tt> method is used,
-    #   the value will not be percent encoded automatically.  Unicode
-    #   normalization will be performed both before and after sending the
-    #   value to the transform method.
+    #   An optional processor object may be supplied.
+    #
+    # The object should respond to either the <tt>validate</tt> or
+    # <tt>transform</tt> messages or both. Both the <tt>validate</tt> and
+    # <tt>transform</tt> methods should take two parameters: <tt>name</tt> and
+    # <tt>value</tt>. The <tt>validate</tt> method should return <tt>true</tt>
+    # or <tt>false</tt>; <tt>true</tt> if the value of the variable is valid,
+    # <tt>false</tt> otherwise. An <tt>InvalidTemplateValueError</tt>
+    # exception will be raised if the value is invalid. The <tt>transform</tt>
+    # method should return the transformed variable value as a <tt>String</tt>.
+    # If a <tt>transform</tt> method is used, the value will not be percent
+    # encoded automatically. Unicode normalization will be performed both
+    # before and after sending the value to the transform method.
     #
     # @return [Addressable::URI] The expanded URI template.
     #
@@ -484,55 +485,79 @@ module Addressable
     #
     # @return [Array] The variables present in the template's pattern.
     def variables
-      @variables ||= (begin
-        result = []
-
-        expansions, expansion_regexp = parse_template_pattern(pattern)
-        expansions.each do |expansion|
-          if expansion =~ OPERATOR_EXPANSION
-            _, _, variables, _ = parse_template_expansion(expansion)
-            result.concat(variables)
-          else
-            result << expansion[VARIABLE_EXPANSION, 1]
-          end
-        end
-        result.uniq
-      end)
+      @variables ||= ordered_variable_defaults.map { |var, val| var }.uniq
     end
     alias_method :keys, :variables
 
+    ##
+    # Returns a mapping of variables to their default values specified
+    # in the template. Variables without defaults are not returned.
+    #
+    # @return [Hash] Mapping of template variables to their defaults
+    def variable_defaults
+      @variable_defaults ||=
+        Hash[*ordered_variable_defaults.reject { |k, v| v.nil? }.flatten]
+    end
+
   private
+    def ordered_variable_defaults
+      @ordered_variable_defaults ||= (begin
+        expansions, expansion_regexp = parse_template_pattern(pattern)
+
+        expansions.inject([]) do |result, expansion|
+          case expansion
+          when OPERATOR_EXPANSION
+            _, _, variables, mapping = parse_template_expansion(expansion)
+            result.concat variables.map { |var| [var, mapping[var]] }
+          when VARIABLE_EXPANSION
+            result << [$1, $2]
+          end
+          result
+        end
+      end)
+    end
+
     ##
     # Transforms a mapping so that values can be substituted into the
     # template.
     #
     # @param [Hash] mapping The mapping of variables to values.
     # @param [#validate, #transform] processor
-    #   An optional processor object may be supplied.  The object should
-    #   respond to either the <tt>validate</tt> or <tt>transform</tt> messages
-    #   or both.  Both the <tt>validate</tt> and <tt>transform</tt> methods
-    #   should take two parameters: <tt>name</tt> and <tt>value</tt>.  The
-    #   <tt>validate</tt> method should return <tt>true</tt> or
-    #   <tt>false</tt>; <tt>true</tt> if the value of the variable is valid,
-    #   <tt>false</tt> otherwise.  An <tt>InvalidTemplateValueError</tt>
-    #   exception will be raised if the value is invalid.  The
-    #   <tt>transform</tt> method should return the transformed variable
-    #   value as a <tt>String</tt>.  If a <tt>transform</tt> method is used,
-    #   the value will not be percent encoded automatically.  Unicode
-    #   normalization will be performed both before and after sending the
-    #   value to the transform method.
+    #   An optional processor object may be supplied.
+    #
+    # The object should respond to either the <tt>validate</tt> or
+    # <tt>transform</tt> messages or both. Both the <tt>validate</tt> and
+    # <tt>transform</tt> methods should take two parameters: <tt>name</tt> and
+    # <tt>value</tt>. The <tt>validate</tt> method should return <tt>true</tt>
+    # or <tt>false</tt>; <tt>true</tt> if the value of the variable is valid,
+    # <tt>false</tt> otherwise. An <tt>InvalidTemplateValueError</tt> exception
+    # will be raised if the value is invalid. The <tt>transform</tt> method
+    # should return the transformed variable value as a <tt>String</tt>. If a
+    # <tt>transform</tt> method is used, the value will not be percent encoded
+    # automatically. Unicode normalization will be performed both before and
+    # after sending the value to the transform method.
     #
     # @return [Hash] The transformed mapping.
     def transform_mapping(mapping, processor=nil)
       return mapping.inject({}) do |accu, pair|
         name, value = pair
+        value = value.to_s if Numeric === value || Symbol === value
+
         unless value.respond_to?(:to_ary) || value.respond_to?(:to_str)
           raise TypeError,
             "Can't convert #{value.class} into String or Array."
         end
 
-        value =
-          value.respond_to?(:to_ary) ? value.to_ary : value.to_str
+        if Symbol === name
+          name = name.to_s
+        elsif name.respond_to?(:to_str)
+          name = name.to_str
+        else
+          raise TypeError,
+            "Can't convert #{name.class} into String."
+        end
+        value = value.respond_to?(:to_ary) ? value.to_ary : value.to_str
+
         # Handle unicode normalization
         if value.kind_of?(Array)
           value.map! { |val| Addressable::IDNA.unicode_normalize_kc(val) }
@@ -775,9 +800,9 @@ module Addressable
     # @return [Array]
     #   A tuple of the operator, argument, variables, and mapping.
     def parse_template_expansion(capture, mapping={})
-      operator, argument, variables = capture[1...-1].split("|")
+      operator, argument, variables = capture[1...-1].split("|", -1)
       operator.gsub!(/^\-/, "")
-      variables = variables.split(",")
+      variables = variables.split(",", -1)
       mapping = (variables.inject({}) do |accu, var|
         varname, _, vardefault = var.scan(/^(.+?)(=(.*))?$/)[0]
         accu[varname] = vardefault
@@ -796,8 +821,8 @@ module Addressable
     # @return [Regexp]
     #   A regular expression which may be used to parse a template pattern.
     def parse_template_pattern(pattern, processor=nil)
-      # Escape the pattern.  The two gsubs restore the escaped curly braces
-      # back to their original form.  Basically, escape everything that isn't
+      # Escape the pattern. The two gsubs restore the escaped curly braces
+      # back to their original form. Basically, escape everything that isn't
       # within an expansion.
       escaped_pattern = Regexp.escape(
         pattern
@@ -819,7 +844,7 @@ module Addressable
             parse_template_expansion(expansion)
           if processor != nil && processor.respond_to?(:match)
             # We can only lookup the match values for single variable
-            # operator expansions.  Besides, ".*" is usually the only
+            # operator expansions. Besides, ".*" is usually the only
             # reasonable value for multivariate operators anyways.
             if ["prefix", "suffix", "list"].include?(operator)
               capture_group = "(#{processor.match(names.first)})"
